@@ -4,7 +4,7 @@
 retroact <- data.frame(
   lists  = rep(0:4, each=6),
   treeid = rep(1:10, rep(c(4, 2), 5)),
-  resp   = rep(c("n1", "n2", "n3", "n4", "m1", "m2"), 5),
+  resp   = rep(c("E1", "E2", "E3", "E4", "F1", "F2"), 5),
   freq   = c(97, 5,  9, 39, 38, 37, 
              71, 2,  6, 71, 24, 51, 
              55, 3, 10, 82, 25, 50,
